@@ -39,9 +39,11 @@ pub fn cli() -> App {
             .value_name("PATH"),
         )
         .arg_manifest_path()
+        .arg_ignore_rust_version()
         .arg_message_format()
         .arg_build_plan()
         .arg_unit_graph()
+        .arg_future_incompat_report()
         .after_help("Run `cargo help build` for more detailed information.\n")
 }
 
@@ -51,7 +53,7 @@ pub fn exec(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
         config,
         CompileMode::Build,
         Some(&ws),
-        ProfileChecking::Checked,
+        ProfileChecking::Custom,
     )?;
 
     if let Some(out_dir) = args.value_of_path("out-dir", config) {
