@@ -28,12 +28,20 @@ pub fn cli() -> App {
         .arg_target_triple("Build for the target triple")
         .arg_target_dir()
         .arg_features()
+        .arg_package_spec_no_all(
+            "Package(s) to assemble",
+            "Assemble all packages in the workspace",
+            "Don't assemble specified packages",
+        )
         .arg_manifest_path()
         .arg_jobs()
+        .after_help("Run `cargo help package` for more detailed information.\n")
 }
 
 pub fn exec(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
     let ws = args.workspace(config)?;
+    let specs = args.packages_from_flags()?;
+
     ops::package(
         &ws,
         &PackageOpts {
@@ -42,12 +50,12 @@ pub fn exec(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
             list: args.is_present("list"),
             check_metadata: !args.is_present("no-metadata"),
             allow_dirty: args.is_present("allow-dirty"),
+            to_package: specs,
             targets: args.targets(),
             jobs: args.jobs()?,
-            features: args._values_of("features"),
-            all_features: args.is_present("all-features"),
-            no_default_features: args.is_present("no-default-features"),
+            cli_features: args.cli_features()?,
         },
     )?;
+
     Ok(())
 }
