@@ -7,7 +7,11 @@ pub fn cli() -> App {
         .about("Remove a pushed crate from the index")
         .arg(opt("quiet", "No output printed to stdout").short("q"))
         .arg(Arg::with_name("crate"))
-        .arg(opt("vers", "The version to yank or un-yank").value_name("VERSION"))
+        .arg(
+            opt("vers", "The version to yank or un-yank")
+                .value_name("VERSION")
+                .required(true),
+        )
         .arg(opt(
             "undo",
             "Undo a yank, putting a version back into the index",
@@ -15,17 +19,7 @@ pub fn cli() -> App {
         .arg(opt("index", "Registry index to yank from").value_name("INDEX"))
         .arg(opt("token", "API token to use when authenticating").value_name("TOKEN"))
         .arg(opt("registry", "Registry to use").value_name("REGISTRY"))
-        .after_help(
-            "\
-The yank command removes a previously pushed crate's version from the server's
-index. This command does not delete any data, and the crate will still be
-available for download via the registry's download link.
-
-Note that existing crates locked to a yanked version will still be able to
-download the yanked version to use it. Cargo will, however, not allow any new
-crates to be locked to any yanked version.
-",
-        )
+        .after_help("Run `cargo help yank` for more detailed information.\n")
 }
 
 pub fn exec(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
