@@ -29,17 +29,16 @@ fn depend_on_alt_registry() {
     Package::new("bar", "0.0.1").alternative(true).publish();
 
     p.cargo("build")
-        .with_stderr(&format!(
+        .with_stderr(
             "\
-[UPDATING] `{reg}` index
+[UPDATING] `alternative` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] bar v0.0.1 (registry `[ROOT][..]`)
-[COMPILING] bar v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] bar v0.0.1 (registry `alternative`)
+[COMPILING] bar v0.0.1 (registry `alternative`)
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
 ",
-            reg = registry::alt_registry_path().to_str().unwrap()
-        ))
+        )
         .run();
 
     p.cargo("clean").run();
@@ -48,7 +47,7 @@ fn depend_on_alt_registry() {
     p.cargo("build")
         .with_stderr(
             "\
-[COMPILING] bar v0.0.1 (registry `[ROOT][..]`)
+[COMPILING] bar v0.0.1 (registry `alternative`)
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
 ",
@@ -83,19 +82,18 @@ fn depend_on_alt_registry_depends_on_same_registry_no_index() {
         .publish();
 
     p.cargo("build")
-        .with_stderr(&format!(
+        .with_stderr(
             "\
-[UPDATING] `{reg}` index
+[UPDATING] `alternative` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] [..] v0.0.1 (registry `[ROOT][..]`)
-[DOWNLOADED] [..] v0.0.1 (registry `[ROOT][..]`)
-[COMPILING] baz v0.0.1 (registry `[ROOT][..]`)
-[COMPILING] bar v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] [..] v0.0.1 (registry `alternative`)
+[DOWNLOADED] [..] v0.0.1 (registry `alternative`)
+[COMPILING] baz v0.0.1 (registry `alternative`)
+[COMPILING] bar v0.0.1 (registry `alternative`)
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
 ",
-            reg = registry::alt_registry_path().to_str().unwrap()
-        ))
+        )
         .run();
 }
 
@@ -126,19 +124,18 @@ fn depend_on_alt_registry_depends_on_same_registry() {
         .publish();
 
     p.cargo("build")
-        .with_stderr(&format!(
+        .with_stderr(
             "\
-[UPDATING] `{reg}` index
+[UPDATING] `alternative` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] [..] v0.0.1 (registry `[ROOT][..]`)
-[DOWNLOADED] [..] v0.0.1 (registry `[ROOT][..]`)
-[COMPILING] baz v0.0.1 (registry `[ROOT][..]`)
-[COMPILING] bar v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] [..] v0.0.1 (registry `alternative`)
+[DOWNLOADED] [..] v0.0.1 (registry `alternative`)
+[COMPILING] baz v0.0.1 (registry `alternative`)
+[COMPILING] bar v0.0.1 (registry `alternative`)
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
 ",
-            reg = registry::alt_registry_path().to_str().unwrap()
-        ))
+        )
         .run();
 }
 
@@ -169,21 +166,19 @@ fn depend_on_alt_registry_depends_on_crates_io() {
         .publish();
 
     p.cargo("build")
-        .with_stderr_unordered(&format!(
+        .with_stderr_unordered(
             "\
-[UPDATING] `{alt_reg}` index
-[UPDATING] `{reg}` index
+[UPDATING] `alternative` index
+[UPDATING] `dummy-registry` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] baz v0.0.1 (registry `[ROOT][..]`)
-[DOWNLOADED] bar v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] baz v0.0.1 (registry `dummy-registry`)
+[DOWNLOADED] bar v0.0.1 (registry `alternative`)
 [COMPILING] baz v0.0.1
-[COMPILING] bar v0.0.1 (registry `[ROOT][..]`)
+[COMPILING] bar v0.0.1 (registry `alternative`)
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
 ",
-            alt_reg = registry::alt_registry_path().to_str().unwrap(),
-            reg = registry::registry_path().to_str().unwrap()
-        ))
+        )
         .run();
 }
 
@@ -403,20 +398,19 @@ fn alt_registry_and_crates_io_deps() {
         .publish();
 
     p.cargo("build")
-        .with_stderr_contains(format!(
-            "[UPDATING] `{}` index",
-            registry::alt_registry_path().to_str().unwrap()
-        ))
-        .with_stderr_contains(&format!(
-            "[UPDATING] `{}` index",
-            registry::registry_path().to_str().unwrap()
-        ))
-        .with_stderr_contains("[DOWNLOADED] crates_io_dep v0.0.1 (registry `[ROOT][..]`)")
-        .with_stderr_contains("[DOWNLOADED] alt_reg_dep v0.1.0 (registry `[ROOT][..]`)")
-        .with_stderr_contains("[COMPILING] alt_reg_dep v0.1.0 (registry `[ROOT][..]`)")
-        .with_stderr_contains("[COMPILING] crates_io_dep v0.0.1")
-        .with_stderr_contains("[COMPILING] foo v0.0.1 ([CWD])")
-        .with_stderr_contains("[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s")
+        .with_stderr_unordered(
+            "\
+[UPDATING] `alternative` index
+[UPDATING] `dummy-registry` index
+[DOWNLOADING] crates ...
+[DOWNLOADED] crates_io_dep v0.0.1 (registry `dummy-registry`)
+[DOWNLOADED] alt_reg_dep v0.1.0 (registry `alternative`)
+[COMPILING] alt_reg_dep v0.1.0 (registry `alternative`)
+[COMPILING] crates_io_dep v0.0.1
+[COMPILING] foo v0.0.1 ([CWD])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
+",
+        )
         .run();
 }
 
@@ -607,7 +601,7 @@ fn patch_alt_reg() {
     p.cargo("build")
         .with_stderr(
             "\
-[UPDATING] `[ROOT][..]` index
+[UPDATING] `alternative` index
 [COMPILING] bar v0.1.0 ([CWD]/bar)
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
@@ -697,24 +691,20 @@ fn no_api() {
         .build();
 
     p.cargo("build")
-        .with_stderr(&format!(
+        .with_stderr(
             "\
-[UPDATING] `{reg}` index
+[UPDATING] `alternative` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] bar v0.0.1 (registry `[ROOT][..]`)
-[COMPILING] bar v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] bar v0.0.1 (registry `alternative`)
+[COMPILING] bar v0.0.1 (registry `alternative`)
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
 ",
-            reg = registry::alt_registry_path().to_str().unwrap()
-        ))
+        )
         .run();
 
     // Check all of the API commands.
-    let err = format!(
-        "[ERROR] registry `{}` does not support API commands",
-        registry::alt_registry_path().display()
-    );
+    let err = "[ERROR] registry `alternative` does not support API commands";
 
     p.cargo("login --registry alternative TOKEN")
         .with_status(101)
@@ -829,6 +819,7 @@ fn alt_reg_metadata() {
                         "publish": null,
                         "authors": [],
                         "categories": [],
+                        "default_run": null,
                         "keywords": [],
                         "readme": null,
                         "repository": null,
@@ -885,6 +876,7 @@ fn alt_reg_metadata() {
                         "publish": null,
                         "authors": [],
                         "categories": [],
+                        "default_run": null,
                         "keywords": [],
                         "readme": null,
                         "repository": null,
@@ -909,6 +901,7 @@ fn alt_reg_metadata() {
                         "publish": null,
                         "authors": [],
                         "categories": [],
+                        "default_run": null,
                         "keywords": [],
                         "readme": null,
                         "repository": null,
@@ -933,6 +926,7 @@ fn alt_reg_metadata() {
                         "publish": null,
                         "authors": [],
                         "categories": [],
+                        "default_run": null,
                         "keywords": [],
                         "readme": null,
                         "repository": null,
@@ -982,6 +976,7 @@ fn alt_reg_metadata() {
                         "publish": null,
                         "authors": [],
                         "categories": [],
+                        "default_run": null,
                         "keywords": [],
                         "readme": null,
                         "repository": null,
@@ -1019,6 +1014,7 @@ fn alt_reg_metadata() {
                         "publish": null,
                         "authors": [],
                         "categories": [],
+                        "default_run": null,
                         "keywords": [],
                         "readme": null,
                         "repository": null,
@@ -1115,6 +1111,7 @@ fn unknown_registry() {
                   "publish": null,
                   "authors": [],
                   "categories": [],
+                  "default_run": null,
                   "keywords": [],
                   "readme": null,
                   "repository": null,
@@ -1139,6 +1136,7 @@ fn unknown_registry() {
                   "publish": null,
                   "authors": [],
                   "categories": [],
+                  "default_run": null,
                   "keywords": [],
                   "readme": null,
                   "repository": null,
@@ -1176,6 +1174,7 @@ fn unknown_registry() {
                   "publish": null,
                   "authors": [],
                   "categories": [],
+                  "default_run": null,
                   "keywords": [],
                   "readme": null,
                   "repository": null,
@@ -1235,17 +1234,16 @@ fn registries_index_relative_url() {
     Package::new("bar", "0.0.1").alternative(true).publish();
 
     p.cargo("build")
-        .with_stderr(&format!(
+        .with_stderr(
             "\
-[UPDATING] `{reg}` index
+[UPDATING] `relative` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] bar v0.0.1 (registry `[ROOT][..]`)
-[COMPILING] bar v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] bar v0.0.1 (registry `relative`)
+[COMPILING] bar v0.0.1 (registry `relative`)
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
 ",
-            reg = registry::alt_registry_path().to_str().unwrap()
-        ))
+        )
         .run();
 }
 
