@@ -15,21 +15,21 @@ fn simple_cross() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.0"
-            authors = []
-            build = "build.rs"
-        "#,
+                [package]
+                name = "foo"
+                version = "0.0.0"
+                authors = []
+                build = "build.rs"
+            "#,
         )
         .file(
             "build.rs",
             &format!(
                 r#"
-            fn main() {{
-                assert_eq!(std::env::var("TARGET").unwrap(), "{}");
-            }}
-        "#,
+                    fn main() {{
+                        assert_eq!(std::env::var("TARGET").unwrap(), "{}");
+                    }}
+                "#,
                 cross_compile::alternate()
             ),
         )
@@ -37,11 +37,11 @@ fn simple_cross() {
             "src/main.rs",
             &format!(
                 r#"
-            use std::env;
-            fn main() {{
-                assert_eq!(env::consts::ARCH, "{}");
-            }}
-        "#,
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -49,10 +49,10 @@ fn simple_cross() {
 
     let target = cross_compile::alternate();
     p.cargo("build -v --target").arg(&target).run();
-    assert!(p.target_bin(&target, "foo").is_file());
+    assert!(p.target_bin(target, "foo").is_file());
 
     if cross_compile::can_run_on_host() {
-        p.process(&p.target_bin(&target, "foo")).run();
+        p.process(&p.target_bin(target, "foo")).run();
     }
 }
 
@@ -67,30 +67,30 @@ fn simple_cross_config() {
             ".cargo/config",
             &format!(
                 r#"
-            [build]
-            target = "{}"
-        "#,
+                    [build]
+                    target = "{}"
+                "#,
                 cross_compile::alternate()
             ),
         )
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.0"
-            authors = []
-            build = "build.rs"
-        "#,
+                [package]
+                name = "foo"
+                version = "0.0.0"
+                authors = []
+                build = "build.rs"
+            "#,
         )
         .file(
             "build.rs",
             &format!(
                 r#"
-            fn main() {{
-                assert_eq!(std::env::var("TARGET").unwrap(), "{}");
-            }}
-        "#,
+                    fn main() {{
+                        assert_eq!(std::env::var("TARGET").unwrap(), "{}");
+                    }}
+                "#,
                 cross_compile::alternate()
             ),
         )
@@ -98,11 +98,11 @@ fn simple_cross_config() {
             "src/main.rs",
             &format!(
                 r#"
-            use std::env;
-            fn main() {{
-                assert_eq!(env::consts::ARCH, "{}");
-            }}
-        "#,
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -110,10 +110,10 @@ fn simple_cross_config() {
 
     let target = cross_compile::alternate();
     p.cargo("build -v").run();
-    assert!(p.target_bin(&target, "foo").is_file());
+    assert!(p.target_bin(target, "foo").is_file());
 
     if cross_compile::can_run_on_host() {
-        p.process(&p.target_bin(&target, "foo")).run();
+        p.process(&p.target_bin(target, "foo")).run();
     }
 }
 
@@ -127,14 +127,14 @@ fn simple_deps() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            path = "../bar"
-        "#,
+                [dependencies.bar]
+                path = "../bar"
+            "#,
         )
         .file("src/main.rs", "extern crate bar; fn main() { bar::bar(); }")
         .build();
@@ -146,10 +146,213 @@ fn simple_deps() {
 
     let target = cross_compile::alternate();
     p.cargo("build --target").arg(&target).run();
-    assert!(p.target_bin(&target, "foo").is_file());
+    assert!(p.target_bin(target, "foo").is_file());
 
     if cross_compile::can_run_on_host() {
-        p.process(&p.target_bin(&target, "foo")).run();
+        p.process(&p.target_bin(target, "foo")).run();
+    }
+}
+
+/// Always take care of setting these so that
+/// `cross_compile::alternate()` is the actually-picked target
+fn per_crate_target_test(
+    default_target: Option<&'static str>,
+    forced_target: Option<&'static str>,
+    arg_target: Option<&'static str>,
+) {
+    if cross_compile::disabled() {
+        return;
+    }
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    cargo-features = ["per-package-target"]
+
+                    [package]
+                    name = "foo"
+                    version = "0.0.0"
+                    authors = []
+                    build = "build.rs"
+                    {}
+                    {}
+                "#,
+                default_target
+                    .map(|t| format!(r#"default-target = "{}""#, t))
+                    .unwrap_or(String::new()),
+                forced_target
+                    .map(|t| format!(r#"forced-target = "{}""#, t))
+                    .unwrap_or(String::new()),
+            ),
+        )
+        .file(
+            "build.rs",
+            &format!(
+                r#"
+                    fn main() {{
+                        assert_eq!(std::env::var("TARGET").unwrap(), "{}");
+                    }}
+                "#,
+                cross_compile::alternate()
+            ),
+        )
+        .file(
+            "src/main.rs",
+            &format!(
+                r#"
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
+                cross_compile::alternate_arch()
+            ),
+        )
+        .build();
+
+    let mut cmd = p.cargo("build -v");
+    if let Some(t) = arg_target {
+        cmd.arg("--target").arg(&t);
+    }
+    cmd.masquerade_as_nightly_cargo().run();
+    assert!(p.target_bin(cross_compile::alternate(), "foo").is_file());
+
+    if cross_compile::can_run_on_host() {
+        p.process(&p.target_bin(cross_compile::alternate(), "foo"))
+            .run();
+    }
+}
+
+#[cargo_test]
+fn per_crate_default_target_is_default() {
+    per_crate_target_test(Some(cross_compile::alternate()), None, None);
+}
+
+#[cargo_test]
+fn per_crate_default_target_gets_overridden() {
+    per_crate_target_test(
+        Some(cross_compile::unused()),
+        None,
+        Some(cross_compile::alternate()),
+    );
+}
+
+#[cargo_test]
+fn per_crate_forced_target_is_default() {
+    per_crate_target_test(None, Some(cross_compile::alternate()), None);
+}
+
+#[cargo_test]
+fn per_crate_forced_target_does_not_get_overridden() {
+    per_crate_target_test(
+        None,
+        Some(cross_compile::alternate()),
+        Some(cross_compile::unused()),
+    );
+}
+
+#[cargo_test]
+fn workspace_with_multiple_targets() {
+    if cross_compile::disabled() {
+        return;
+    }
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [workspace]
+                members = ["native", "cross"]
+            "#,
+        )
+        .file(
+            "native/Cargo.toml",
+            r#"
+                cargo-features = ["per-package-target"]
+
+                [package]
+                name = "native"
+                version = "0.0.0"
+                authors = []
+                build = "build.rs"
+            "#,
+        )
+        .file(
+            "native/build.rs",
+            &format!(
+                r#"
+                    fn main() {{
+                        assert_eq!(std::env::var("TARGET").unwrap(), "{}");
+                    }}
+                "#,
+                cross_compile::native()
+            ),
+        )
+        .file(
+            "native/src/main.rs",
+            &format!(
+                r#"
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
+                cross_compile::native_arch()
+            ),
+        )
+        .file(
+            "cross/Cargo.toml",
+            &format!(
+                r#"
+                    cargo-features = ["per-package-target"]
+
+                    [package]
+                    name = "cross"
+                    version = "0.0.0"
+                    authors = []
+                    build = "build.rs"
+                    default-target = "{}"
+                "#,
+                cross_compile::alternate(),
+            ),
+        )
+        .file(
+            "cross/build.rs",
+            &format!(
+                r#"
+                    fn main() {{
+                        assert_eq!(std::env::var("TARGET").unwrap(), "{}");
+                    }}
+                "#,
+                cross_compile::alternate()
+            ),
+        )
+        .file(
+            "cross/src/main.rs",
+            &format!(
+                r#"
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
+                cross_compile::alternate_arch()
+            ),
+        )
+        .build();
+
+    let mut cmd = p.cargo("build -v");
+    cmd.masquerade_as_nightly_cargo().run();
+
+    assert!(p.bin("native").is_file());
+    assert!(p.target_bin(cross_compile::alternate(), "cross").is_file());
+
+    p.process(&p.bin("native")).run();
+    if cross_compile::can_run_on_host() {
+        p.process(&p.target_bin(cross_compile::alternate(), "cross"))
+            .run();
     }
 }
 
@@ -165,9 +368,9 @@ fn linker() {
             ".cargo/config",
             &format!(
                 r#"
-            [target.{}]
-            linker = "my-linker-tool"
-        "#,
+                    [target.{}]
+                    linker = "my-linker-tool"
+                "#,
                 target
             ),
         )
@@ -176,11 +379,11 @@ fn linker() {
             "src/foo.rs",
             &format!(
                 r#"
-            use std::env;
-            fn main() {{
-                assert_eq!(env::consts::ARCH, "{}");
-            }}
-        "#,
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -193,7 +396,7 @@ fn linker() {
             "\
 [COMPILING] foo v0.5.0 ([CWD])
 [RUNNING] `rustc --crate-name foo src/foo.rs [..]--crate-type bin \
-    --emit=[..]link -C debuginfo=2 \
+    --emit=[..]link[..]-C debuginfo=2 \
     -C metadata=[..] \
     --out-dir [CWD]/target/{target}/debug/deps \
     --target {target} \
@@ -220,23 +423,23 @@ fn plugin_with_extra_dylib_dep() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            path = "../bar"
-        "#,
+                [dependencies.bar]
+                path = "../bar"
+            "#,
         )
         .file(
             "src/main.rs",
             r#"
-            #![feature(plugin)]
-            #![plugin(bar)]
+                #![feature(plugin)]
+                #![plugin(bar)]
 
-            fn main() {}
-        "#,
+                fn main() {}
+            "#,
         )
         .build();
     let _bar = project()
@@ -244,34 +447,34 @@ fn plugin_with_extra_dylib_dep() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "bar"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "bar"
+                version = "0.0.1"
+                authors = []
 
-            [lib]
-            name = "bar"
-            plugin = true
+                [lib]
+                name = "bar"
+                plugin = true
 
-            [dependencies.baz]
-            path = "../baz"
-        "#,
+                [dependencies.baz]
+                path = "../baz"
+            "#,
         )
         .file(
             "src/lib.rs",
             r#"
-            #![feature(plugin_registrar, rustc_private)]
+                #![feature(rustc_private)]
 
-            extern crate baz;
-            extern crate rustc_driver;
+                extern crate baz;
+                extern crate rustc_driver;
 
-            use rustc_driver::plugin::Registry;
+                use rustc_driver::plugin::Registry;
 
-            #[plugin_registrar]
-            pub fn foo(reg: &mut Registry) {
-                println!("{}", baz::baz());
-            }
-        "#,
+                #[no_mangle]
+                pub fn __rustc_plugin_registrar(reg: &mut Registry) {
+                    println!("{}", baz::baz());
+                }
+            "#,
         )
         .build();
     let _baz = project()
@@ -279,15 +482,15 @@ fn plugin_with_extra_dylib_dep() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "baz"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "baz"
+                version = "0.0.1"
+                authors = []
 
-            [lib]
-            name = "baz"
-            crate_type = ["dylib"]
-        "#,
+                [lib]
+                name = "baz"
+                crate_type = ["dylib"]
+            "#,
         )
         .file("src/lib.rs", "pub fn baz() -> i32 { 1 }")
         .build();
@@ -306,27 +509,27 @@ fn cross_tests() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            authors = []
-            version = "0.0.0"
+                [project]
+                name = "foo"
+                authors = []
+                version = "0.0.0"
 
-            [[bin]]
-            name = "bar"
-        "#,
+                [[bin]]
+                name = "bar"
+            "#,
         )
         .file(
             "src/bin/bar.rs",
             &format!(
                 r#"
-            #[allow(unused_extern_crates)]
-            extern crate foo;
-            use std::env;
-            fn main() {{
-                assert_eq!(env::consts::ARCH, "{}");
-            }}
-            #[test] fn test() {{ main() }}
-        "#,
+                    #[allow(unused_extern_crates)]
+                    extern crate foo;
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                    #[test] fn test() {{ main() }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -334,10 +537,10 @@ fn cross_tests() {
             "src/lib.rs",
             &format!(
                 r#"
-            use std::env;
-            pub fn foo() {{ assert_eq!(env::consts::ARCH, "{}"); }}
-            #[test] fn test_foo() {{ foo() }}
-        "#,
+                    use std::env;
+                    pub fn foo() {{ assert_eq!(env::consts::ARCH, "{}"); }}
+                    #[test] fn test_foo() {{ foo() }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -350,8 +553,8 @@ fn cross_tests() {
             "\
 [COMPILING] foo v0.0.0 ([CWD])
 [FINISHED] test [unoptimized + debuginfo] target(s) in [..]
-[RUNNING] target/{triple}/debug/deps/foo-[..][EXE]
-[RUNNING] target/{triple}/debug/deps/bar-[..][EXE]",
+[RUNNING] [..] (target/{triple}/debug/deps/foo-[..][EXE])
+[RUNNING] [..] (target/{triple}/debug/deps/bar-[..][EXE])",
             triple = target
         ))
         .with_stdout_contains("test test_foo ... ok")
@@ -369,18 +572,18 @@ fn no_cross_doctests() {
         .file(
             "src/lib.rs",
             r#"
-            //! ```
-            //! extern crate foo;
-            //! assert!(true);
-            //! ```
-        "#,
+                //! ```
+                //! extern crate foo;
+                //! assert!(true);
+                //! ```
+            "#,
         )
         .build();
 
     let host_output = "\
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] test [unoptimized + debuginfo] target(s) in [..]
-[RUNNING] target/debug/deps/foo-[..][EXE]
+[RUNNING] [..] (target/debug/deps/foo-[..][EXE])
 [DOCTEST] foo
 ";
 
@@ -395,7 +598,7 @@ fn no_cross_doctests() {
             "\
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] test [unoptimized + debuginfo] target(s) in [..]
-[RUNNING] target/{triple}/debug/deps/foo-[..][EXE]
+[RUNNING] [..] (target/{triple}/debug/deps/foo-[..][EXE])
 [DOCTEST] foo
 ",
             triple = target
@@ -448,11 +651,11 @@ fn simple_cargo_run() {
             "src/main.rs",
             &format!(
                 r#"
-            use std::env;
-            fn main() {{
-                assert_eq!(env::consts::ARCH, "{}");
-            }}
-        "#,
+                    use std::env;
+                    fn main() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -473,36 +676,36 @@ fn cross_with_a_build_script() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.0"
-            authors = []
-            build = 'build.rs'
-        "#,
+                [package]
+                name = "foo"
+                version = "0.0.0"
+                authors = []
+                build = 'build.rs'
+            "#,
         )
         .file(
             "build.rs",
             &format!(
                 r#"
-            use std::env;
-            use std::path::PathBuf;
-            fn main() {{
-                assert_eq!(env::var("TARGET").unwrap(), "{0}");
-                let mut path = PathBuf::from(env::var_os("OUT_DIR").unwrap());
-                assert_eq!(path.file_name().unwrap().to_str().unwrap(), "out");
-                path.pop();
-                assert!(path.file_name().unwrap().to_str().unwrap()
-                            .starts_with("foo-"));
-                path.pop();
-                assert_eq!(path.file_name().unwrap().to_str().unwrap(), "build");
-                path.pop();
-                assert_eq!(path.file_name().unwrap().to_str().unwrap(), "debug");
-                path.pop();
-                assert_eq!(path.file_name().unwrap().to_str().unwrap(), "{0}");
-                path.pop();
-                assert_eq!(path.file_name().unwrap().to_str().unwrap(), "target");
-            }}
-        "#,
+                    use std::env;
+                    use std::path::PathBuf;
+                    fn main() {{
+                        assert_eq!(env::var("TARGET").unwrap(), "{0}");
+                        let mut path = PathBuf::from(env::var_os("OUT_DIR").unwrap());
+                        assert_eq!(path.file_name().unwrap().to_str().unwrap(), "out");
+                        path.pop();
+                        assert!(path.file_name().unwrap().to_str().unwrap()
+                                    .starts_with("foo-"));
+                        path.pop();
+                        assert_eq!(path.file_name().unwrap().to_str().unwrap(), "build");
+                        path.pop();
+                        assert_eq!(path.file_name().unwrap().to_str().unwrap(), "debug");
+                        path.pop();
+                        assert_eq!(path.file_name().unwrap().to_str().unwrap(), "{0}");
+                        path.pop();
+                        assert_eq!(path.file_name().unwrap().to_str().unwrap(), "target");
+                    }}
+                "#,
                 target
             ),
         )
@@ -536,25 +739,25 @@ fn build_script_needed_for_host_and_target() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.0"
-            authors = []
-            build = 'build.rs'
+                [package]
+                name = "foo"
+                version = "0.0.0"
+                authors = []
+                build = 'build.rs'
 
-            [dependencies.d1]
-            path = "d1"
-            [build-dependencies.d2]
-            path = "d2"
-        "#,
+                [dependencies.d1]
+                path = "d1"
+                [build-dependencies.d2]
+                path = "d2"
+            "#,
         )
         .file(
             "build.rs",
             r#"
-            #[allow(unused_extern_crates)]
-            extern crate d2;
-            fn main() { d2::d2(); }
-        "#,
+                #[allow(unused_extern_crates)]
+                extern crate d2;
+                fn main() { d2::d2(); }
+            "#,
         )
         .file(
             "src/main.rs",
@@ -567,35 +770,35 @@ fn build_script_needed_for_host_and_target() {
         .file(
             "d1/Cargo.toml",
             r#"
-            [package]
-            name = "d1"
-            version = "0.0.0"
-            authors = []
-            build = 'build.rs'
-        "#,
+                [package]
+                name = "d1"
+                version = "0.0.0"
+                authors = []
+                build = 'build.rs'
+            "#,
         )
         .file("d1/src/lib.rs", "pub fn d1() {}")
         .file(
             "d1/build.rs",
             r#"
-            use std::env;
-            fn main() {
-                let target = env::var("TARGET").unwrap();
-                println!("cargo:rustc-flags=-L /path/to/{}", target);
-            }
-        "#,
+                use std::env;
+                fn main() {
+                    let target = env::var("TARGET").unwrap();
+                    println!("cargo:rustc-flags=-L /path/to/{}", target);
+                }
+            "#,
         )
         .file(
             "d2/Cargo.toml",
             r#"
-            [package]
-            name = "d2"
-            version = "0.0.0"
-            authors = []
+                [package]
+                name = "d2"
+                version = "0.0.0"
+                authors = []
 
-            [dependencies.d1]
-            path = "../d1"
-        "#,
+                [dependencies.d1]
+                path = "../d1"
+            "#,
         )
         .file(
             "d2/src/lib.rs",
@@ -644,14 +847,14 @@ fn build_deps_for_the_right_arch() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.0"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.0"
+                authors = []
 
-            [dependencies.d2]
-            path = "d2"
-        "#,
+                [dependencies.d2]
+                path = "d2"
+            "#,
         )
         .file("src/main.rs", "extern crate d2; fn main() {}")
         .file("d1/Cargo.toml", &basic_manifest("d1", "0.0.0"))
@@ -659,15 +862,15 @@ fn build_deps_for_the_right_arch() {
         .file(
             "d2/Cargo.toml",
             r#"
-            [package]
-            name = "d2"
-            version = "0.0.0"
-            authors = []
-            build = "build.rs"
+                [package]
+                name = "d2"
+                version = "0.0.0"
+                authors = []
+                build = "build.rs"
 
-            [build-dependencies.d1]
-            path = "../d1"
-        "#,
+                [build-dependencies.d1]
+                path = "../d1"
+            "#,
         )
         .file("d2/build.rs", "extern crate d1; fn main() {}")
         .file("d2/src/lib.rs", "")
@@ -687,40 +890,40 @@ fn build_script_only_host() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.0"
-            authors = []
-            build = "build.rs"
+                [package]
+                name = "foo"
+                version = "0.0.0"
+                authors = []
+                build = "build.rs"
 
-            [build-dependencies.d1]
-            path = "d1"
-        "#,
+                [build-dependencies.d1]
+                path = "d1"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .file("build.rs", "extern crate d1; fn main() {}")
         .file(
             "d1/Cargo.toml",
             r#"
-            [package]
-            name = "d1"
-            version = "0.0.0"
-            authors = []
-            build = "build.rs"
-        "#,
+                [package]
+                name = "d1"
+                version = "0.0.0"
+                authors = []
+                build = "build.rs"
+            "#,
         )
         .file("d1/src/lib.rs", "pub fn d1() {}")
         .file(
             "d1/build.rs",
             r#"
-            use std::env;
+                use std::env;
 
-            fn main() {
-                assert!(env::var("OUT_DIR").unwrap().replace("\\", "/")
-                                           .contains("target/debug/build/d1-"),
-                        "bad: {:?}", env::var("OUT_DIR"));
-            }
-        "#,
+                fn main() {
+                    assert!(env::var("OUT_DIR").unwrap().replace("\\", "/")
+                                               .contains("target/debug/build/d1-"),
+                            "bad: {:?}", env::var("OUT_DIR"));
+                }
+            "#,
         )
         .build();
 
@@ -737,16 +940,16 @@ fn plugin_build_script_right_arch() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            build = "build.rs"
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+                build = "build.rs"
 
-            [lib]
-            name = "foo"
-            plugin = true
-        "#,
+                [lib]
+                name = "foo"
+                plugin = true
+            "#,
         )
         .file("build.rs", "fn main() {}")
         .file("src/lib.rs", "")
@@ -778,15 +981,15 @@ fn build_script_with_platform_specific_dependencies() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            build = "build.rs"
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+                build = "build.rs"
 
-            [build-dependencies.d1]
-            path = "d1"
-        "#,
+                [build-dependencies.d1]
+                path = "d1"
+            "#,
         )
         .file(
             "build.rs",
@@ -801,14 +1004,14 @@ fn build_script_with_platform_specific_dependencies() {
             "d1/Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "d1"
-            version = "0.0.0"
-            authors = []
+                    [package]
+                    name = "d1"
+                    version = "0.0.0"
+                    authors = []
 
-            [target.{}.dependencies]
-            d2 = {{ path = "../d2" }}
-        "#,
+                    [target.{}.dependencies]
+                    d2 = {{ path = "../d2" }}
+                "#,
                 host
             ),
         )
@@ -851,18 +1054,18 @@ fn platform_specific_dependencies_do_not_leak() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            build = "build.rs"
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+                build = "build.rs"
 
-            [dependencies.d1]
-            path = "d1"
+                [dependencies.d1]
+                path = "d1"
 
-            [build-dependencies.d1]
-            path = "d1"
-        "#,
+                [build-dependencies.d1]
+                path = "d1"
+            "#,
         )
         .file("build.rs", "extern crate d1; fn main() {}")
         .file("src/lib.rs", "")
@@ -870,14 +1073,14 @@ fn platform_specific_dependencies_do_not_leak() {
             "d1/Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "d1"
-            version = "0.0.0"
-            authors = []
+                    [package]
+                    name = "d1"
+                    version = "0.0.0"
+                    authors = []
 
-            [target.{}.dependencies]
-            d2 = {{ path = "../d2" }}
-        "#,
+                    [target.{}.dependencies]
+                    d2 = {{ path = "../d2" }}
+                "#,
                 host
             ),
         )
@@ -906,18 +1109,18 @@ fn platform_specific_variables_reflected_in_build_scripts() {
             "Cargo.toml",
             &format!(
                 r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            build = "build.rs"
+                    [package]
+                    name = "foo"
+                    version = "0.0.1"
+                    authors = []
+                    build = "build.rs"
 
-            [target.{host}.dependencies]
-            d1 = {{ path = "d1" }}
+                    [target.{host}.dependencies]
+                    d1 = {{ path = "d1" }}
 
-            [target.{target}.dependencies]
-            d2 = {{ path = "d2" }}
-        "#,
+                    [target.{target}.dependencies]
+                    d2 = {{ path = "d2" }}
+                "#,
                 host = host,
                 target = target
             ),
@@ -926,22 +1129,22 @@ fn platform_specific_variables_reflected_in_build_scripts() {
             "build.rs",
             &format!(
                 r#"
-            use std::env;
+                    use std::env;
 
-            fn main() {{
-                let platform = env::var("TARGET").unwrap();
-                let (expected, not_expected) = match &platform[..] {{
-                    "{host}" => ("DEP_D1_VAL", "DEP_D2_VAL"),
-                    "{target}" => ("DEP_D2_VAL", "DEP_D1_VAL"),
-                    _ => panic!("unknown platform")
-                }};
+                    fn main() {{
+                        let platform = env::var("TARGET").unwrap();
+                        let (expected, not_expected) = match &platform[..] {{
+                            "{host}" => ("DEP_D1_VAL", "DEP_D2_VAL"),
+                            "{target}" => ("DEP_D2_VAL", "DEP_D1_VAL"),
+                            _ => panic!("unknown platform")
+                        }};
 
-                env::var(expected).ok()
-                    .expect(&format!("missing {{}}", expected));
-                env::var(not_expected).err()
-                    .expect(&format!("found {{}}", not_expected));
-            }}
-        "#,
+                        env::var(expected).ok()
+                            .expect(&format!("missing {{}}", expected));
+                        env::var(not_expected).err()
+                            .expect(&format!("found {{}}", not_expected));
+                    }}
+                "#,
                 host = host,
                 target = target
             ),
@@ -950,26 +1153,26 @@ fn platform_specific_variables_reflected_in_build_scripts() {
         .file(
             "d1/Cargo.toml",
             r#"
-            [package]
-            name = "d1"
-            version = "0.0.0"
-            authors = []
-            links = "d1"
-            build = "build.rs"
-        "#,
+                [package]
+                name = "d1"
+                version = "0.0.0"
+                authors = []
+                links = "d1"
+                build = "build.rs"
+            "#,
         )
         .file("d1/build.rs", r#"fn main() { println!("cargo:val=1") }"#)
         .file("d1/src/lib.rs", "")
         .file(
             "d2/Cargo.toml",
             r#"
-            [package]
-            name = "d2"
-            version = "0.0.0"
-            authors = []
-            links = "d2"
-            build = "build.rs"
-        "#,
+                [package]
+                name = "d2"
+                version = "0.0.0"
+                authors = []
+                links = "d2"
+                build = "build.rs"
+            "#,
         )
         .file("d2/build.rs", r#"fn main() { println!("cargo:val=1") }"#)
         .file("d2/src/lib.rs", "")
@@ -993,61 +1196,61 @@ fn cross_test_dylib() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [lib]
-            name = "foo"
-            crate_type = ["dylib"]
+                [lib]
+                name = "foo"
+                crate_type = ["dylib"]
 
-            [dependencies.bar]
-            path = "bar"
-        "#,
+                [dependencies.bar]
+                path = "bar"
+            "#,
         )
         .file(
             "src/lib.rs",
             r#"
-            extern crate bar as the_bar;
+                extern crate bar as the_bar;
 
-            pub fn bar() { the_bar::baz(); }
+                pub fn bar() { the_bar::baz(); }
 
-            #[test]
-            fn foo() { bar(); }
-        "#,
+                #[test]
+                fn foo() { bar(); }
+            "#,
         )
         .file(
             "tests/test.rs",
             r#"
-            extern crate foo as the_foo;
+                extern crate foo as the_foo;
 
-            #[test]
-            fn foo() { the_foo::bar(); }
-        "#,
+                #[test]
+                fn foo() { the_foo::bar(); }
+            "#,
         )
         .file(
             "bar/Cargo.toml",
             r#"
-            [package]
-            name = "bar"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "bar"
+                version = "0.0.1"
+                authors = []
 
-            [lib]
-            name = "bar"
-            crate_type = ["dylib"]
-        "#,
+                [lib]
+                name = "bar"
+                crate_type = ["dylib"]
+            "#,
         )
         .file(
             "bar/src/lib.rs",
             &format!(
                 r#"
-             use std::env;
-             pub fn baz() {{
-                assert_eq!(env::consts::ARCH, "{}");
-            }}
-        "#,
+                     use std::env;
+                     pub fn baz() {{
+                        assert_eq!(env::consts::ARCH, "{}");
+                    }}
+                "#,
                 cross_compile::alternate_arch()
             ),
         )
@@ -1060,10 +1263,59 @@ fn cross_test_dylib() {
 [COMPILING] bar v0.0.1 ([CWD]/bar)
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] test [unoptimized + debuginfo] target(s) in [..]
-[RUNNING] target/{arch}/debug/deps/foo-[..][EXE]
-[RUNNING] target/{arch}/debug/deps/test-[..][EXE]",
+[RUNNING] [..] (target/{arch}/debug/deps/foo-[..][EXE])
+[RUNNING] [..] (target/{arch}/debug/deps/test-[..][EXE])",
             arch = cross_compile::alternate()
         ))
         .with_stdout_contains_n("test foo ... ok", 2)
         .run();
 }
+
+#[cargo_test]
+fn doctest_xcompile_linker() {
+    if cross_compile::disabled() {
+        return;
+    }
+    if !is_nightly() {
+        // -Zdoctest-xcompile is unstable
+        return;
+    }
+
+    let target = cross_compile::alternate();
+    let p = project()
+        .file(
+            ".cargo/config",
+            &format!(
+                r#"
+                    [target.{}]
+                    linker = "my-linker-tool"
+                "#,
+                target
+            ),
+        )
+        .file("Cargo.toml", &basic_manifest("foo", "0.1.0"))
+        .file(
+            "src/lib.rs",
+            r#"
+                /// ```
+                /// assert_eq!(1, 1);
+                /// ```
+                pub fn foo() {}
+            "#,
+        )
+        .build();
+
+    // Fails because `my-linker-tool` doesn't actually exist.
+    p.cargo("test --doc -v -Zdoctest-xcompile --target")
+        .arg(&target)
+        .with_status(101)
+        .masquerade_as_nightly_cargo()
+        .with_stderr_contains(&format!(
+            "\
+[RUNNING] `rustdoc --crate-type lib --crate-name foo --test [..]\
+    --target {target} [..] -C linker=my-linker-tool[..]
+",
+            target = target,
+        ))
+        .run();
+}
